import os
import sys

def append_to_name(source_path, suffix):
    dest_path_prefix = source_path + suffix
    dest_path = dest_path_prefix
    dest_index = 0
    while os.path.lexists(dest_path):
        dest_index += 1
        dest_path = dest_path_prefix + str(dest_index)
    os.rename(source_path, dest_path)

def renames(source, destination, override = True, backup = False, backup_suffix = "~"):
    if os.path.exists(destination):
        if override:
            if backup:
                append_to_name(destination, backup_suffix)
            elif os.path.isdir(destination) and not os.path.islink(destination):
                raise OSError("may not override directory '%s'" % destination)
        else:
            raise OSError("path '%s' already exists" % destination)
    os.renames(source, destination)

def symlink(source, link_name, relative = False, override = False, backup = True, backup_suffix = "~"):
    if relative:
        source = os.path.relpath(source, os.path.dirname(link_name))
    # os.path.lexists() returns True if path refers to an existing path and 
    # True for broken symbolic links. 
    if not os.path.lexists(link_name):
        os.symlink(source, link_name)
    elif not os.path.islink(link_name) or os.readlink(link_name) != source:
        if override:
            if backup:
                append_to_name(link_name, backup_suffix)
            else:
                if os.path.isdir(link_name):
                    os.rmdir(link_name)
                else:
                    os.remove(link_name)
            os.symlink(source, link_name)
        else:
            raise OSError("link's path '%s' already exists." % link_name)

def update_locate_database(root_path = None, database_path = None, require_visibility = None):
    import subprocess
    cmd = ['updatedb']
    if root_path:
        cmd.append('--database-root')
        cmd.append(root_path)
    if database_path:
        cmd.append('--output')
        cmd.append(database_path)
    if require_visibility is not None:
        cmd.append('--require-visibility')
        cmd.append('yes' if require_visibility else 'no')
    subprocess.check_call(cmd)

def locate(patterns, match_all = False, ignore_case = False, database_path = None):
    if type(patterns) is not list:
        patterns = [str(patterns)]
    patterns = [str(p) for p in patterns]
    options = []
    if match_all:
        options.append("--all")
    if ignore_case:
        options.append("--ignore-case")
    if database_path:
        options.append('--database')
        options.append(database_path)
    import subprocess
    try:
        output = subprocess.check_output(["locate"] + options + patterns)
    except subprocess.CalledProcessError as e:
        if e.returncode == 1:
            return []
        else:
            raise e
    return output.decode(sys.getfilesystemencoding()).strip().split('\n')
